#target "InDesign"
#targetengine "PhotoBookLayout"		

// -------------------------------------------------------------------------------------------------
/*
	1) Photobook_Layout.jsx - Generate a grid of graphic content rectangles within margins or
	   selected rectangle.
	
	2) Photobook_Library.jsx - Generate a library of multiple layouts.
	  
	Install in:	
		CS2 - Adobe_InDesign_CS2_Install_Folder\Presets\Scripts
		CS3 - Adobe_InDesign_CS3_Install_Folder\Scripts\Scripts Panel
		CS5 - Adobe_InDesign_CS5_Install_Folder\Scripts\Scripts Panel
                CC  - Adobe_InDesign_CC_Install_Folder\Scripts\Scripts Panel
		
	Make a copy of "Photobook_Layout.jsx" and rename copy "Photobook_Library.jsx".
	
	Remove or comment out the #targetengine line above for CS2
	
	Release:	1.0 - 	20 March 2008
                        1.0.1 - 23 March 2008 - add caption frames, guides, open lib test, options persistence
                        1.0.2 - 26 March 2008 - change caption height & gap to fixed
                        1.0.3 - 31 March 2008 - add scale and alignment
                        1.0.4 - 4 Apr 2008 - add scale and alignment to multipane and library gen.
                        1.0.5 - 8 Apr 2008 - library work - add aspect limits, remove lib constraint table, generate in loop
                        1.0.6 - 10 Apr 2008 - clean up code
                        1.0.7 - 4 Nov 2010 - test on CS5
                        1.0.8 - 17 Nov 2011 - Convert menus to two columns for better fit on short screens, add generalized bleeds.
                        1.0.9 - 14 Jan 2013 - Make experimental (*_exp) scripts current. Remove circle-c copyright symbol.
	
        Copyright 2011 Bill Wetzel 18-Nov-2011
        All rights reserved.
*/
// *************************************************************************************************
// Select script operation by filename of script. 

var mk_layout = 0, mk_library = 0;

if( app.activeScript.name == "Photobook_Layout.jsx" )  mk_layout = 1;
// if( app.activeScript.name == "Photobook_Layout_exp.jsx" )  mk_layout = 1;       /// WRW 17 Nov 2011 Testing, 14 Jan 2014 removed
if( app.activeScript.name == "Photobook_Library.jsx" ) mk_library = 1;
// if( app.activeScript.name == "Photobook_Library_exp.jsx" ) mk_library = 1;      /// WRW 17 Nov 2011 Testing, 14 Jan 2014 removed

// -------------------------------------------------------------------------------------------------
//	Initialize menus to default values on first invocation

var initialized;
if( ! initialized ) {
	var ip = new init_params();
	var ilp = new init_library_params();
	
	var ha_t = [ "l", "nl", "c", "nr", "r" ];	
	var va_t = [ "t", "nt", "c", "nb", "b" ];
	var ha_d = [ "left", "near left", "center", "near right", "right" ];
	var va_d = [ "top", "near top", "center", "near bottom", "bottom" ];
		
	initialized = 1;
}

// -------------------------------------------------------------------------------------------------
// Must have a document open. Script obtains page dimensions, bleeds, and margins from document

var myError;

try{ app.activeDocument } 
catch( myError ) {
	alert( "You must have a document open to run this script.\n" + myError );
	exit(1);	
	}

// -------------------------------------------------------------------------------------------------
//	Save measurement units and set to points for compatibility with measurementEditBox, which works in points.
//	Disable redraw.  Sadly, we must deal with version specifics in this one small area.

var version = app.version.split( "." )[0];	// CS5 - 7, CS3 - 5, CS2 - 4

if( version >= 5 )var orig_script_redraw = app.scriptPreferences.enableRedraw;	
var orig_hori_measurement_units = app.activeDocument.viewPreferences.horizontalMeasurementUnits;
var orig_vert_measurement_units = app.activeDocument.viewPreferences.verticalMeasurementUnits;

if( version >= 5 ) app.scriptPreferences.enableRedraw = false;		// Disable redraw, appeared to be corrupting screen.
app.activeDocument.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
app.activeDocument.viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
	
// -------------------------------------------------------------------------------------------------
// 	Figure out what to do.

     if( mk_layout ) 	do_mk_layout();
else if( mk_library ) 	do_mk_library();
else                    alert( "No operation selected." );

// -------------------------------------------------------------------------------------------------
//	Final cleanup. Restore measurement units.

app.activeDocument.viewPreferences.horizontalMeasurementUnits = orig_hori_measurement_units;
app.activeDocument.viewPreferences.verticalMeasurementUnits = orig_vert_measurement_units;
if( version >= 5 ) app.scriptPreferences.enableRedraw = orig_script_redraw;

exit(0);	// Exit from script, all done.

// *************************************************************************************************
//	Make one layout according to data provided by user.       // if( pa !== false ) {	// Works in CS3, not CS2

function do_mk_layout() {

	var pa = get_params();
	if( pa == false ) return;		
	var items = new Array();
	var pg = get_page( pa );
	
	if( pa.pane_dual ) 	draw_dualpane_layout( pa, pg, items );
	else 				draw_layout( pa, pg, null, items ); 
	
	// Put layout in group and select group if more than one item in layout.
	if( items.length > 1 ) {
		var myGroup = pg.page.groups.add( items );
		myGroup.select();
	}
}

// *************************************************************************************************
//	Make a library of layouts with constraints provided by user.

function do_mk_library() {
	var myDoc = app.activeDocument;	
	var lib_file;
	
	var got_filename = false;
	do {	// Get the library file name.
		lib_file = File.saveDialog( "Enter Photobook Library file name", "Files:*.indt" );	
		if( lib_file === null ) return;	// User canceled out of dialog
		if( lib_file.exists === true ) {	// File exists, ask overwrite confirmation
			var myDialog = app.dialogs.add({name:"Confirm Overwrite"});		
			with(myDialog) {
				with( dialogColumns.add() ) {   //Add a dialog column.
						staticTexts.add({staticLabel:"File: \" " + lib_file.fsName + "\" exists, do you want to overwrite? "} );	
				}
			}
			got_filename = myDialog.show();
			myDialog.destroy();
		} else {	// File doesn't exist. Use filename given by user.
			got_filename = true;
		}
	} while( ! got_filename );
		
	try {			
		var myLib = app.libraries.add( lib_file );	// Create a library file
	} catch( myError ) {
		alert( "Please close the library: " + lib_file + " and try again.\n" + myError );
		return;
	}
		
	var lpa = get_library_params();				// Get parameters from user
	if( lpa == false ) return;					// User cancelled out, 	if( lpa !== false ) {	// NG in CS3
		
	var myPage = myDoc.pages.add();				// Add a working page, will remove when done.
	app.activeWindow.activePage = myPage;
	if( lpa.single_pane ) do_single_pane_lib( lpa, myLib );	// Make each of the three components of the library under control of user parms.
	if( lpa.dual_pane )   do_dual_pane_lib( lpa, myLib );
	if( lpa.bleed )       do_bleed_lib( lpa, myLib );

	myPage.remove();							// Remove the temporary page used for building the libraries
}

// *************************************************************************************************

function do_single_pane_lib( lpa, myLib ) {  
	
	var am_t = [ "F", "AL", "AP", "S" ];
	var am_d = [ "Full", "Landscape", "Portrait", "Square" ];
		
	var pg = get_page( lpa );		// Get page parameters
	
	for( var am_i = 0; am_i < 4; am_i++ ) {	
	    if( ! lpa.shape[ am_i ] ) continue;
		
		for( var ha_i = 0; ha_i < 5; ha_i++ ) {
			if( ! lpa.halign[ha_i] ) continue;
			
			for( var va_i = 0; va_i < 5; va_i++ ) {
				if( ! lpa.valign[va_i] ) continue;
				
				for( var row = 1; row <= lpa.single_maxrows; row++ ) {
				
					for( var col = 1; col <= lpa.single_maxcols; col++ ) {
						
						var pa = { 	aspect_mode: am_i, 
									aspect_long: lpa.aspect_long,
									aspect_short: lpa.aspect_short,
									rows: row,
									cols: col,
									fixed: lpa.fixed,				// percent gap
									gap_fixed: lpa.gap_fixed,
									gap_rel: lpa.gap_rel, 			// Gap in percent
									caption: lpa.caption,
									caption_y: lpa.caption_y,
									caption_gap_y: lpa.caption_gap_y,
									scale_pc: lpa.scale_pc,
									near_margin_pc: lpa.near_margin_pc,
									halign: ha_i,
									valign: va_i
								};
								
						draw_outline_box( pg ); 			// Draw a page outline box first so it is on the bottom.		
						draw_layout( pa, pg, null, null );	// Draw a layout on page	
						
						store_and_clear( myLib, pg.page, 
										am_t[am_i] + " " + row + "x" + col + " " + ha_t[ha_i] + "-" + va_t[va_i], 	// Name
										am_d[am_i] + ", " + row + " rows, " + col + " cols, " + ha_d[ha_i] + ", " + va_d[va_i]   // Description
										);
					}
				}
			}
		}
	}
}

// -------------------------------------------------------------------------------------------------
//	Dual-Pane - Terms Top/Bottom, Left/Right refer to location of primary or larger pane

function do_dual_pane_lib( lpa, myLib ) {  	
	var orient_name = [ "h", "v" ];
	var orient_desc = [ "Hori", "Vert" ];
	var split_name_h = [ "r", "e", "e", "l" ];	// Large on Right, Equal, Left
	var split_name_v = [ "b", "e", "e", "t" ];	// Large on Bottom, Equal, Top
	var split_desc_h = [ "Right", "Even", "Even", "Left" ];	// Large on
	var split_desc_v = [ "Bottom", "Even", "Even", "Top" ];	// Large on 
	var split_ratio = [ .333, .5, .5, .666 ];	// Value is size of first pane, second pane is remainder
	
	var pg = get_page( lpa );		// Get page parameters
	
	for( var ha_i = 0; ha_i < 5; ha_i++ ) {
		if( ! lpa.halign[ha_i] ) continue;
		
		for( var va_i = 0; va_i < 5; va_i++ ) {
			if( ! lpa.valign[va_i] ) continue;
			
			var rows_a, cols_a;
			var rows_b, cols_b;
			
			for( var orient_i = 0; orient_i < 2; orient_i++ ) {
				if( ! lpa.orient[ orient_i ] ) continue;
				
				// Note: two even splits for grid in left & right pane.
				for( var split_i = 0; split_i < 4; split_i++ ) {
					if( ! lpa.split[ split_i ] ) continue;
					
					for( var rc = 1; rc <= lpa.dual_b_max; rc++ ) {
						if( split_i == 2 && rc == 1 ) continue;	// Skip redundant 1x1 even layout
					
						// ----------------------------------------------------------
						switch( orient_i ) {
						case 0:	// horizontal split
							switch( split_i ) {
								case 0:	// Single on right
								case 1: // equal single on left
									var t = { rows_a: rc, cols_a: 1, rows_b: 1, cols_b: 1 };
									break;
								case 2: // equal single on right
									var t = { rows_a: 1, cols_a: 1, rows_b: rc, cols_b: 1 };
									break;
								case 3: // Single on left
									var t = { rows_a: 1, cols_a: 1, rows_b: rc, cols_b: 1 };
									break;
							}
							break;
						// ----------------------------------------------------------
						case 1: // vertical split
							switch( split_i ) {
								case 0:	// Single on bottom
									var t = { rows_a: 1, cols_a: rc, rows_b: 1, cols_b: 1 };
									break;
								case 1: // equal, single on bottom
									var t = { rows_a: 1, cols_a: rc, rows_b: 1, cols_b: 1 };
									break;
								case 2: // equal, single on top
								case 3:	// Single on top
									var t = { rows_a: 1, cols_a: 1, rows_b: 1, cols_b: rc };
									break;
							}
							break;
							// ----------------------------------------------------------
						}
						// -------------------------------------------------------------------------------------------------
						
						var pa = { 	
							cols: t.cols_a, 
							rows: t.rows_a,
							pane_cols: t.cols_b,
							pane_rows: t.rows_b,
							pane_split: split_ratio[split_i],
							pane_orient: orient_i,
							fixed: lpa.fixed,
							gap_fixed: lpa.gap_fixed, 	
							gap_rel: lpa.gap_rel, 		// Gap in percent
							caption: lpa.caption,
							caption_y: lpa.caption_y,
							caption_gap_y: lpa.caption_gap_y,
							scale_pc: lpa.scale_pc,
							near_margin_pc: lpa.near_margin_pc,
							halign: ha_i,
							valign: va_i
						};
						
						draw_outline_box( pg ); 	// Draw a page outline box first so it is on the bottom.
						var aspect_limits = draw_dualpane_layout( pa, pg, null );	// Draw a layout on page	
						
						// Add page to library only if axpect of every drawn box is within limits.
						if( aspect_limits.min_land > lpa.min_land && aspect_limits.max_port < lpa.max_port ) {
						
							// -------------------------------------------------------------------------------------------------
							//	Makes some labels for the name and description of the library asset
							var t1, t2, t3, t4;
							
							if( !orient_i ) {	// Horizontal
								t1 = t.rows_a;
								t2 = t.rows_b;
								t3 = split_desc_h[split_i];
								t4 = split_name_h[split_i];
							}
							else {			// Vertical
								t1 = t.cols_a;
								t2 = t.cols_b;
								t3 = split_desc_v[split_i];
								t4 = split_name_v[split_i];
							}
							
							page_store( myLib, pg.page, 
										"x" +     orient_name[orient_i] + 
										" " + t4 + " " + t1 + "x" + t2 + 
										" " + ha_t[ha_i] + "-" + va_t[va_i],
										"Dual "	+ orient_desc[orient_i] + ", " + t3 + 
										" " + t1 + "x" + t2 + ", " + ha_d[ha_i] + ", " + va_d[va_i]
										);
							// -------------------------------------------------------------------------------------------------
						}
						page_clear( pg.page );
					}
				}
			}
		}			
	}
}		

// -------------------------------------------------------------------------------------------------
//	Full bleed

function do_bleed_lib( lpa, myLib ) {  
	var pg = get_page( lpa );		
	var myDoc = app.activeDocument;	
	
	var myRect = pg.page.rectangles.add({geometricBounds:[ 	
		-myDoc.documentPreferences.documentBleedTopOffset, 
		-myDoc.documentPreferences.documentBleedInsideOrLeftOffset, 
		myDoc.documentPreferences.pageHeight + myDoc.documentPreferences.documentBleedBottomOffset, 
		myDoc.documentPreferences.pageWidth + myDoc.documentPreferences.documentBleedOutsideOrRightOffset 
		]});
	myRect.contentType = ContentType.graphicType;
	myRect.frameFittingOptions.fittingOnEmptyFrame = EmptyFrameFittingOptions.fillProportionally;
	// myRect.fit(FitOptions.proportionally);	/// Test for CS5
	
	store_and_clear( myLib, pg.page, "Z 1x1", "Bleed 1x1" );	// store myPage in myLib with names formed from row, col, t2, t2, and description from d1	

}

// *************************************************************************************************
//	Bisected to support skipping the lib store on aspect limits exceeded

function store_and_clear( lib, page, name, description ) {
	page_store( lib, page, name, description );
	page_clear( page );
}

// -------------------------------------------------------------------------------------------------
// Store page in library	
function page_store( lib, page, name, description ) {
	var asset = lib.store( page.pageItems );			
	asset.name = name;	// Give it a name and description
	asset.description = description;
}

// -------------------------------------------------------------------------------------------------
// Remove all of the items drawn
function page_clear( page ) {	
	for( var i = page.pageItems.length - 1; i >= 0; i-- ) page.pageItems[0].remove();	
}
		
// *************************************************************************************************	
// Draw an outline box.
// Draw it for all since Aspect and Square need to be aligned too. Better to be consistent.
// Draw it as full bleed. May be useful to layout a full bleed. More useful than just full page or within margins.
						
function draw_outline_box( pg ) {
	var myDoc = app.activeDocument;	
	var myRect = pg.page.rectangles.add({geometricBounds:[ 	
		-myDoc.documentPreferences.documentBleedTopOffset, 
		-myDoc.documentPreferences.documentBleedInsideOrLeftOffset, 
		myDoc.documentPreferences.pageHeight + myDoc.documentPreferences.documentBleedBottomOffset, 
		myDoc.documentPreferences.pageWidth + myDoc.documentPreferences.documentBleedOutsideOrRightOffset 
		]});
	myRect.contentType = ContentType.unassigned;
	// myRect.contentType = ContentType.graphicType;	///
	myRect.frameFittingOptions.fittingOnEmptyFrame = EmptyFrameFittingOptions.fillProportionally;  // Make it useable.
	// myRect.fit(FitOptions.proportionally);	/// Test for CS5
}						
						
// *************************************************************************************************							
// Get area on which to draw layout
// Return: working_left, working_top - location on page to draw layout
//            layout_x, layout_y - size of layout
//            page - active document and active page.

function get_page( pa ) {
	var pg = new Object();
	
	var myDocument = app.activeDocument;
	
	var myPage = app.activeWindow.activePage;			
	pg.page = myPage;
	// not used now pg.spread = app.activeWindow.activeSpread;

	var page_y = myDocument.documentPreferences.pageHeight;
	var page_x = myDocument.documentPreferences.pageWidth;
	
	var margin_left = myPage.marginPreferences.left;
	var margin_right = myPage.marginPreferences.right;
	
	var margin_top = myPage.marginPreferences.top;
	var margin_bottom = myPage.marginPreferences.bottom;	
	
	var page_offset_x = myPage.bounds[1];   // left bounds of page within spread

	// -------------------------------------------------------------------------------------------------
	// Use selection dimensions if a rectangle is selected
	
	var found_sel = 0;

	if( myDocument.selection.length == 1 ) {
		var sel = myDocument.selection[0];
		
		if( sel.constructor.name == "Rectangle" ) {
			var bounds = sel.geometricBounds;
			found_sel = 1;
			var sel_top = bounds[0];
			var sel_left = bounds[1];
			var sel_bottom = bounds[2];
			var sel_right = bounds[3];
			
			pg.working_left = sel_left;
			pg.working_top = sel_top;
			pg.layout_y = (sel_bottom - sel_top) * pa.scale_pc/100;
			pg.layout_x = (sel_right - sel_left) * pa.scale_pc/100;	
			
			pg.align_space_y = sel_bottom - sel_top;
			pg.align_space_x = sel_right - sel_left;
		}	
	}
	// -------------------------------------------------------------------------------------------------
	// Otherwise use full area within margins
	
	if( ! found_sel ) {
			if( 0 ) {	// Keep this if'ed out to show what worked before generalized bleeds.
				pg.working_left = margin_left + page_offset_x;
				pg.working_top = margin_top;
				
				pg.layout_y = ( page_y - (margin_top + margin_bottom ) ) * pa.scale_pc/100;
				pg.layout_x = ( page_x - (margin_left + margin_right )  ) * pa.scale_pc/100;
				
				pg.align_space_y = page_y - (margin_top + margin_bottom );
				pg.align_space_x = page_x - (margin_left + margin_right );
			}
			
			// -------------------------------------------------------------------------
			//	WRW 17 Nov 2011 - Add generalized bleeds.
			
			else {
				myDoc = myDocument;
				
				pg.align_space_y = page_y;
				pg.align_space_x = page_x;
				
				pg.layout_y = page_y * pa.scale_pc/100;
				pg.layout_x = page_x * pa.scale_pc/100;
				
				// - - - - - - - - - - - - - - -
				// TOP
				
				if( pa.bleedTop ) {
					pg.working_top = -myDoc.documentPreferences.documentBleedTopOffset;
					pg.align_space_y += myDoc.documentPreferences.documentBleedTopOffset;
					pg.layout_y += myDoc.documentPreferences.documentBleedTopOffset * pa.scale_pc/100;
				}
				else {
					pg.working_top = margin_top;
					pg.align_space_y -= margin_top;
					pg.layout_y -= margin_top * pa.scale_pc/100;
				}
				
				// - - - - - - - - - - - - - - -
				// RIGHT
				
				if( pa.bleedRight ) {
					pg.align_space_x += myDoc.documentPreferences.documentBleedOutsideOrRightOffset;
					pg.layout_x += myDoc.documentPreferences.documentBleedOutsideOrRightOffset * pa.scale_pc/100;
				}
				else {
					pg.align_space_x -= margin_right;
					pg.layout_x -= margin_right * pa.scale_pc/100;
				}
				
				// - - - - - - - - - - - - - - -
				// BOTTOM
				
				if( pa.bleedBottom ) {
					pg.align_space_y += myDoc.documentPreferences.documentBleedBottomOffset;
					pg.layout_y += myDoc.documentPreferences.documentBleedBottomOffset * pa.scale_pc/100;
				}
				else {
					pg.align_space_y -= margin_bottom;
					pg.layout_y -= margin_bottom * pa.scale_pc/100;
				}
				
				// - - - - - - - - - - - - - - -
				// LEFT
				
				if( pa.bleedLeft ) {
					pg.working_left = page_offset_x - myDoc.documentPreferences.documentBleedInsideOrLeftOffset;
					pg.align_space_x += myDoc.documentPreferences.documentBleedInsideOrLeftOffset;
					pg.layout_x += myDoc.documentPreferences.documentBleedInsideOrLeftOffset * pa.scale_pc/100;
				}
				else {
					pg.working_left = page_offset_x + margin_left;
					pg.align_space_x -= margin_left;
					pg.layout_x -= margin_left * pa.scale_pc/100;
				}			
			}
			// -------------------------------------------------------------------------
	}
		
	// -------------------------------------------------------------------------------------------------	
	pg.near_margin = Math.sqrt( pg.align_space_x * pg.align_space_x + 
								pg.align_space_y * pg.align_space_y ) * pa.near_margin_pc/100;
	
	return pg;
}

// *************************************************************************************************
// gap is used if provided, otherwise it is computed
// Return aspect ratio (y/x) of photo after aspect & caption adjustments.

function draw_layout( pa, pg, gap, items ){
	var cols = pa.cols;
	var rows = pa.rows;
	var guides = pa.guides;	
	
	// -----------------------------------------------------------------------------------	
	
	if( gap === null ) gap = get_gap( cols, rows, pg.layout_x, pg.layout_y, pa.gap_fixed, pa.gap_rel, pa.fixed );	
	cell_x = (pg.layout_x - (cols-1) * gap)/cols;
	cell_y = (pg.layout_y - (rows-1) * gap)/rows;

	// -----------------------------------------------------------------------------------
	// Include a space below the graphic frame for a caption text frame.
	
	if( pa.caption ) {
		var caption_y = pa.caption_y;
		var caption_gap_y = pa.caption_gap_y;
		cell_y -= (caption_y + caption_gap_y);
	}	
	// -----------------------------------------------------------------------------------
	// Compute photo_x & y with consideration for aspect_mode
	
	var photo_x, photo_y;
	var adjust = 0;
	var photo_aspect;
	
	switch( pa.aspect_mode ) {
	case 0:		// Full
		photo_x = cell_x;
		photo_y = cell_y;
		photo_aspect = photo_y/photo_x;
		break;
		
	case 1: 	// Aspect Land
		photo_aspect = pa.aspect_short/pa.aspect_long;
		adjust = 1;
		break;
		
	case 2:		// Aspect Port
		photo_aspect = pa.aspect_long/pa.aspect_short;
		adjust = 1;
		break;
		
	case 3:		// Square
		photo_aspect = 1;
		adjust = 1;
		break;
	}
	
	// -----------------------------------------------------------------------------------
	
	if( adjust ) {
		var cell_aspect = cell_y/cell_x;
		if( cell_aspect < photo_aspect ) {
			photo_y = cell_y;
			photo_x = photo_y/photo_aspect;	
		}
		else {
			photo_x = cell_x;
			photo_y = photo_x * photo_aspect;		
		}
	}
	
	// -----------------------------------------------------------------------------------
	var top, left, bottom, right;
	
	left = align( pa.halign, 
					pg.working_left, 
					pg.align_space_x, 
					photo_x * cols + gap * (cols - 1), 					
					pg.near_margin );
					
	for( var x = 0; x < cols; x++ ) {
	
		top = align( pa.valign, 
					pg.working_top, 
					pg.align_space_y, 
					photo_y * rows + gap * (rows - 1), 					
					pg.near_margin );
					
		
		for( var y = 0; y < rows; y++ ) {
		
			right = left + photo_x;
			bottom = top + photo_y;
			
			// ------------------------------------
			if( guides ) {
				if( x == 0 ) {
					var myGuide = pg.page.guides.add();
					// Guide on spread var myGuide = app.activeWindow.activeSpread.guides.add();
					myGuide.orientation = HorizontalOrVertical.horizontal;
					myGuide.location = top;	
					
					myGuide = pg.page.guides.add();
					myGuide.orientation = HorizontalOrVertical.horizontal;
					myGuide.location = bottom;	
				}
				if( y == 0 ) {
					var myGuide = pg.page.guides.add();
					myGuide.orientation = HorizontalOrVertical.vertical;
					myGuide.location = left;
					
					myGuide = pg.page.guides.add();
					myGuide.orientation = HorizontalOrVertical.vertical;
					myGuide.location = right;
				}
			}
	
			// ------------------------------------
			//	Draw graphics rectangle
			
			var myRect = pg.page.rectangles.add({geometricBounds:[ top, left, bottom, right ]});		
			myRect.contentType = ContentType.graphicType;
			myRect.frameFittingOptions.fittingOnEmptyFrame = EmptyFrameFittingOptions.fillProportionally; 
			// myRect.fit(FitOptions.proportionally);	/// Test for CS5
			if( items != null ) items.push( myRect );
			
			top += photo_y;
			
			// ------------------------------------
			// Optionally draw caption text frame
			
			if( pa.caption ) {
				top += caption_gap_y;	// Gap between photo and caption
				bottom = top + caption_y;
				var myText = pg.page.textFrames.add({geometricBounds:[ top, left, bottom, right ]});	
				if( items != null ) items.push( myText );
						
				if( guides ) {			// Draw horizontal guides for caption frame.
					if( x == 0 ) {		// Don't need to draw vertical guides again, same as done above.
						var myGuide = pg.page.guides.add();
						// Guide on spread var myGuide = app.activeWindow.activeSpread.guides.add();
						myGuide.orientation = HorizontalOrVertical.horizontal;
						myGuide.location = top;	
						
						myGuide = pg.page.guides.add();
						myGuide.orientation = HorizontalOrVertical.horizontal;
						myGuide.location = bottom;	
					}
				}
				top = bottom;
			}
			top += gap;
		}
		left += photo_x + gap;
	}
	return photo_aspect;
}

// *************************************************************************************************
// No Guides with dual-pane. Would look too congested. 
// Return aspect_limits

function draw_dualpane_layout( pa, pg, items ) {

	var aspect_limits = new alimits();
	var aspect;
	
	var layout_x = pg.layout_x;
	var layout_y = pg.layout_y;
	
	var cols1 = pa.cols;
	var rows1 = pa.rows;
	var cols2 = pa.pane_cols;
	var rows2 = pa.pane_rows;
	
	var my_pa;
	
	// Make local copy of pg to modify. 
	var my_pg = new Object;	
	var i;
	for( i in pg ) my_pg[i] = pg[i];
	
	var pane_gap1, pane_gap2, pane_gap;	
	var split = pa.pane_split; // First pane, second is remainder.
	
	// -----------------------------------------------------------------------------------
		
	my_pg.working_left = align( pa.halign, 
								my_pg.working_left, 
								pg.align_space_x, 
								pg.layout_x, 					
								pg.near_margin );
	
	my_pg.working_top = align( pa.valign, 
								my_pg.working_top, 
								pg.align_space_y, 
								pg.layout_y, 					
								pg.near_margin );
	
	// -----------------------------------------------------------------------------------
		
	if( ! pa.pane_orient ) { 	//Horizontal Split, left pane.
		
		// The pane_gap is the smaller of the x or y gaps in the layout within either pane
		// Note, the gap computation is not 100% correct because the layout_x value is not reduced by the gap, but the error is irrevalent. 
		
		pane_gap1 = get_gap( cols1, rows1, pg.layout_x * split, pg.layout_y, pa.gap_fixed, pa.gap_rel, pa.fixed );
		pane_gap2 = get_gap( cols2, rows2, pg.layout_x * (1 - split), pg.layout_y, pa.gap_fixed, pa.gap_rel, pa.fixed );		
		pane_gap = pane_gap1 > pane_gap2 ? pane_gap2 : pane_gap1; 
		
		my_pg.layout_x = layout_x * split - pane_gap/2;
	
		my_pa = { 	aspect_mode: 0, 	// Dual pane always max size
					rows: rows1,
					cols: cols1,
					caption: pa.caption,
					caption_y: pa.caption_y,
					caption_gap_y: pa.caption_gap_y
					
				};
				
		aspect = draw_layout( my_pa, my_pg, pane_gap, items );
		aspect_limits.test( aspect );
		
		// Right pane
		my_pg.working_left += my_pg.layout_x + pane_gap;
		my_pg.layout_x = pg.layout_x * (1-split) - pane_gap/2;
		
		my_pa = { 	aspect_mode: 0,	// Dual pane is always full size 
					rows: rows2,
					cols: cols2,
					caption: pa.caption,
					caption_y: pa.caption_y,
					caption_gap_y: pa.caption_gap_y
				};
				
		aspect = draw_layout( my_pa, my_pg, pane_gap, items );
		aspect_limits.test( aspect );
	} 
	
	// -----------------------------------------------------------------------------------------------------------------------------------------------------------
	
	else { 	// Vertical Split, top pane
		 
		pane_gap1 = get_gap( cols1, rows1, pg.layout_x, pg.layout_y * split, pa.gap_fixed, pa.gap_rel, pa.fixed );
		pane_gap2 = get_gap( cols2, rows2, pg.layout_x, pg.layout_y * (1 - split), pa.gap_fixed, pa.gap_rel, pa.fixed );		
		pane_gap = pane_gap1 > pane_gap2 ? pane_gap2 : pane_gap1; 
		
		my_pg.layout_y = layout_y * split - pane_gap/2;
		
		// -----------------------------------------------------------------------------------
		
		my_pa = { 	aspect_mode: 0, 
					rows: rows1,
					cols: cols1,
					caption: pa.caption,
					caption_y: pa.caption_y,
					caption_gap_y: pa.caption_gap_y
				};
				
		aspect = draw_layout( my_pa, my_pg, pane_gap, items );
		aspect_limits.test( aspect );
		
		// Bottom pane
		my_pg.working_top += my_pg.layout_y + pane_gap;
		my_pg.layout_y = pg.layout_y * (1-split) - pane_gap/2;
		
		my_pa = { 	aspect_mode: 0, 
					rows: rows2,
					cols: cols2,
					caption: pa.caption,
					caption_y: pa.caption_y,
					caption_gap_y: pa.caption_gap_y
				};
				
		aspect = draw_layout( my_pa, my_pg, pane_gap, items );
		aspect_limits.test( aspect );
	}
	return aspect_limits;
}

// *************************************************************************************************
// Return gap in doc units given layout parameters and gap in fraction. Min of hori and vert gap.

function get_gap( cols, rows, size_x, size_y, gap_fixed, gap_rel, fixed ) {
	var gx, gy, ret;
	
	if( fixed ) ret = gap_fixed;
	else { 
		gx = size_x / cols * gap_rel/100;
		gy = size_y / rows * gap_rel/100;
		ret = gx > gy ? gy : gx;
	}

	return ret;
}

// *************************************************************************************************
//	Align one dimension of object within container 

function align( mode, position, container, object, near_margin ) {

	var min = position;
	var max = position + container - object;
	
	switch( mode ) {
	case 0:		// Left
		break;
	case 1:		// Near left
		position += near_margin;	
		break;
	case 2:		// Mid
		position += container/2 - object/2;
		break;
	case 3:		// Near right
		position += container - object - near_margin;
		break;
	case 4:		// Right
		position += container - object;
		break;
	}
	
	if( position < min ) position = min;
	if( position > max ) position = max;
	return position;
}
// *************************************************************************************************
//	Deal with range of aspect ratios for limits on library generation.

function test_aspect( aspect ) {
	if( aspect > 1 ) { 	// portrait
		if( aspect > this.max_port ) this.max_port = aspect;
	}
	if( aspect < 1 ) { // landscape
		if( aspect < this.min_land ) this.min_land = aspect;
	}
}
// *************************************************************************************************

function alimits() {
	this.test = test_aspect;
	this.min_land = 1;
	this.max_port = 1;
}

// *************************************************************************************************
// Get parameters from user for one layout.

function get_params() {
	var r;
	var myDialog = app.dialogs.add({name:"Photo Book Layout Builder"});
	
	with(myDialog) {
		with( dialogColumns.add() ) {   // Add a dialog column.
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) {   // Need another withing a border panel to add rows.
					with( dialogRows.add () ) {						
						staticTexts.add({staticLabel:"Single/First (top/left):  " } );			
						staticTexts.add({staticLabel:"Rows:" } );
						var rowsEditbox = realEditboxes.add({ editValue: ip.rows, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
						staticTexts.add({staticLabel:"Columns:" } );
						var colsEditbox = realEditboxes.add( { editValue: ip.cols, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
					}						
					with( dialogRows.add () ) {	
						var guides_CheckBox = checkboxControls.add( { staticLabel: "Draw Guides", checkedState: ip.guides } );	
					}
				}
			}
			
			// -----------------------------------------------
			// WRW 18 Nov 2011 - Add selective bleed
			
			with( borderPanels.add() ) {
				with( dialogColumns.add() ) {
					with( dialogRows.add() ) {
						staticTexts.add( { staticLabel:"Bleed: " } );
						var bleedTop_CheckBox = checkboxControls.add( { staticLabel: "Top", checkedState: ip.bleedTop } );
						var bleedRight_CheckBox = checkboxControls.add( { staticLabel: "Right", checkedState: ip.bleedRight } );
						var bleedBottom_CheckBox = checkboxControls.add( { staticLabel: "Bottom", checkedState: ip.bleedBottom } );
						var bleedLeft_CheckBox = checkboxControls.add( { staticLabel: "Left", checkedState: ip.bleedLeft } );
					}
				}
			}
			
			// -----------------------------------------------
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Layout scale (% available): " } );
						var scale_pc_Editbox = percentEditboxes.add({	editValue: ip.scale_pc, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});	
					}

					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Near margin (% diagonal): " } );
						var near_margin_pc_Editbox = percentEditboxes.add({	editValue: ip.near_margin_pc, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});	
					}
					
					with( dialogRows.add () ) {	
						with( dialogColumns.add() ) { 
							with( dialogRows.add () ) {	
								staticTexts.add({staticLabel:"Hori Alignment:"});
								var halignButtonGroup = radiobuttonGroups.add();
								halignButtonGroup.radiobuttonControls.add( {staticLabel: "left"});					
								halignButtonGroup.radiobuttonControls.add( {staticLabel: "near left" });
								halignButtonGroup.radiobuttonControls.add( {staticLabel: "center" });
								halignButtonGroup.radiobuttonControls.add( {staticLabel: "near right" });
								halignButtonGroup.radiobuttonControls.add( {staticLabel: "right" });
								halignButtonGroup.selectedButton = ip.halign;
							}
						}
						with( dialogColumns.add() ) { 
							with( dialogRows.add () ) {	
								staticTexts.add({staticLabel:"Vert Alignment:"});
								var valignButtonGroup = radiobuttonGroups.add();
								valignButtonGroup.radiobuttonControls.add( {staticLabel: "top"});					
								valignButtonGroup.radiobuttonControls.add( {staticLabel: "near top" });
								valignButtonGroup.radiobuttonControls.add( {staticLabel: "center" });
								valignButtonGroup.radiobuttonControls.add( {staticLabel: "near bottom" });
								valignButtonGroup.radiobuttonControls.add( {staticLabel: "bottom" });
								valignButtonGroup.selectedButton = ip.valign;
							}
						}
					}
				}
			}
			with(borderPanels.add( )) {	
				with( dialogColumns.add() ) { 	
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Gap between cells:" } );
					}
					with( dialogRows.add () ) {			
						var gapButtonGroup = radiobuttonGroups.add();				
						gapButtonGroup.radiobuttonControls.add( {staticLabel: "Relative to cell size" });
						gapButtonGroup.radiobuttonControls.add( {staticLabel: "Constant" });
						gapButtonGroup.selectedButton = ip.fixed;
					}
					with( dialogRows.add () ) {								
						staticTexts.add({staticLabel:"Relative (% of cell size):"});
						var gap_rel_Editbox    = percentEditboxes.add({	editValue: ip.gap_rel, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Constant (measurement units):"});
						var gap_fixed_Editbox = measurementEditboxes.add({ editUnits: orig_hori_measurement_units,
																			editValue: ip.gap_fixed, 
																			minWidth: 100, 
																			minimumValue: 0, 
																			maximumValue: 100
																		});			
					}
				}
			}
		}
		
		// WRW 17 Nov 2011 Convert to two columns for better fit on short screens.
		
		with( dialogColumns.add() ) {   // Add a dialog column. 
			
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 						
					with( dialogRows.add () ) {							
						staticTexts.add({staticLabel:"Shape:"});
						var aspectButtonGroup = radiobuttonGroups.add();
						aspectButtonGroup.radiobuttonControls.add( {staticLabel: "Full"});					
						aspectButtonGroup.radiobuttonControls.add( {staticLabel: "Aspect Land" });
						aspectButtonGroup.radiobuttonControls.add( {staticLabel: "Aspect Port" });
						aspectButtonGroup.radiobuttonControls.add( {staticLabel: "Square" });
						aspectButtonGroup.selectedButton = ip.aspect_mode;
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Aspect Long:"});
						var aspect_long_Editbox = realEditboxes.add({editValue: ip.aspect_long});
						staticTexts.add({staticLabel:"Short:"});
						var aspect_short_Editbox = realEditboxes.add({editValue: ip.aspect_short});
					}
				}
			}
			
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 						
					with( dialogRows.add () ) {	
						var caption_CheckBox = checkboxControls.add( { staticLabel: "Include Caption Text Frame", checkedState: ip.caption } );	
					}
				
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Caption Height:"});
						var caption_y_Editbox = measurementEditboxes.add({	editValue: ip.caption_y, 
																			minimumValue: 0, 
																			maximumValue: 100, 
																			minWidth: 50});
					}
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Caption Gap:"});
						var caption_gap_y_Editbox = measurementEditboxes.add({	editValue: ip.caption_gap_y, 
																				minimumValue: 0, 
																				maximumValue: 100, 
																				minWidth: 50});					
					}
					
				}
			}
			
			with(borderPanels.add( )) {
			
				with( dialogColumns.add() ) { 						
					with( dialogRows.add () ) {		
						var pane_dualCheckBox = checkboxControls.add( { staticLabel: "Split Pane", checkedState: ip.pane_dual } );
					}
					
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Second (bottom/right):  " } );			
						staticTexts.add({staticLabel:"Rows:" } );
						var panerowsEditbox = realEditboxes.add({ editValue : ip.pane_rows, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
						staticTexts.add({staticLabel:"Columns:" } );
						var panecolsEditbox = realEditboxes.add( { editValue : ip.pane_cols, minimumValue: 1, maximumValue: 25, minWidth: 30 } );	
					}

					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Split:   " } );
						var pane_orientButtonGroup = radiobuttonGroups.add();
						pane_orientButtonGroup.radiobuttonControls.add( {staticLabel: "Horizontal    " }); // 0
						pane_orientButtonGroup.radiobuttonControls.add( {staticLabel: "Vertical"  });  // 1
						pane_orientButtonGroup.selectedButton = ip.pane_orient;
					}
					
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Second Pane:   " } );
						var paneButtonGroup = radiobuttonGroups.add();
						paneButtonGroup.radiobuttonControls.add( {staticLabel: "1/3" }); // 0
						paneButtonGroup.radiobuttonControls.add( {staticLabel: "equal" });  // 1
						paneButtonGroup.radiobuttonControls.add( {staticLabel: "2/3" });	// 2
						paneButtonGroup.radiobuttonControls.add( {staticLabel: "Custom" });	// 3
						paneButtonGroup.selectedButton = ip.pane_split_button;
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Custom split:" } );
						var pane_customEditbox = realEditboxes.add( { editValue : ip.pane_custom, minimumValue: 0, maximumValue: 1, minWidth: 40 } );
					}
				}
			}
		}
	}
	
	// -----------------------------------------------------------------------------------
	
	var myResult = myDialog.show();
	
	if( myResult === true ) {
		var res = new Object();
		res.aspect_mode = aspectButtonGroup.selectedButton;
		res.aspect_long = aspect_long_Editbox.editValue;
		res.aspect_short = aspect_short_Editbox.editValue;
		res.rows = rowsEditbox.editValue;
		res.cols = colsEditbox.editValue;
		res.scale_pc = scale_pc_Editbox.editValue;
		res.near_margin_pc = near_margin_pc_Editbox.editValue;
		res.halign = halignButtonGroup.selectedButton;
		res.valign = valignButtonGroup.selectedButton;
		res.gap_fixed = gap_fixed_Editbox.editValue;
		res.gap_rel = gap_rel_Editbox.editValue;
		res.fixed = gapButtonGroup.selectedButton;
		
		res.caption = caption_CheckBox.checkedState;

		res.caption_y = caption_y_Editbox.editValue;
		res.caption_gap_y = caption_gap_y_Editbox.editValue;
		
		res.pane_dual = pane_dualCheckBox.checkedState;				// 0 single, 1 dual
		res.pane_orient = pane_orientButtonGroup.selectedButton;	// 0 Horizontal, 1 Vertical
		
		switch( paneButtonGroup.selectedButton ) {
			case 0:	res.pane_split = 0.666; break;
			case 1: res.pane_split = 0.5;   break;
			case 2: res.pane_split = 0.333; break;
			case 3: res.pane_split = 1 - pane_customEditbox.editValue; break;		
		}
		res.pane_split_button = paneButtonGroup.selectedButton;
		res.pane_custom = pane_customEditbox.editValue;
		res.pane_rows = panerowsEditbox.editValue;
		res.pane_cols = panecolsEditbox.editValue;
		
		res.guides = guides_CheckBox.checkedState;	
		res.bleedTop = bleedTop_CheckBox.checkedState;
		res.bleedRight = bleedRight_CheckBox.checkedState;
		res.bleedBottom = bleedBottom_CheckBox.checkedState;
		res.bleedLeft = bleedLeft_CheckBox.checkedState;

		var t;
		for( t in res ) ip[t] = res[t];
		r = ip;
		
	}
	else {
		r = false;
	}

	myDialog.destroy();   //Remove the dialog box from memory.
	return r;

} // End get_params()

// *************************************************************************************************
//	Get user input for building library

function get_library_params() {
	var r;
	var myDialog = app.dialogs.add({name:"Photo Book Library Builder"});
	
	with(myDialog) {
		with( dialogColumns.add() ) {   //Add a dialog column.
			with(borderPanels.add()) {
				with( dialogColumns.add() ) {   // Need another withing a border panel to add rows.
					with( dialogRows.add () ) {
						var single_CheckBox = checkboxControls.add( { staticLabel: "Single Pane", checkedState: ilp.single_pane } );	
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Maximum:" } );			
						staticTexts.add({staticLabel:"Rows:" } );
						var single_maxrows_Editbox = realEditboxes.add({ editValue : ilp.single_maxrows, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
						staticTexts.add({staticLabel:"Columns:" } );
						var single_maxcols_Editbox = realEditboxes.add( { editValue : ilp.single_maxcols, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
					}
					with( dialogRows.add () ) {	
						var shape = new Array();
						staticTexts.add({staticLabel:"Shapes:   " } );		
						shape[0] = checkboxControls.add( { staticLabel: "Full   ", checkedState: ilp.shape[0] } );	
						shape[1] = checkboxControls.add( { staticLabel: "Aspect Land   ", checkedState: ilp.shape[1] } );	
						shape[2] = checkboxControls.add( { staticLabel: "Aspect Port   ", checkedState: ilp.shape[2] } );
						shape[3] = checkboxControls.add( { staticLabel: "Square", checkedState: ilp.shape[3] } );	
					}
				}
			}
			
			with(borderPanels.add()){
				with( dialogColumns.add() ) {  
					with( dialogRows.add () ) {			
						var dual_CheckBox = checkboxControls.add( { staticLabel: "Split Pane", checkedState: ilp.dual_pane } );	
					}
					/*
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"First (left/top): Maximum:" } );			
						staticTexts.add({staticLabel:"Rows:" } );
						var dual_a_maxrows_Editbox = realEditboxes.add({ editValue : ilp.dual_a_maxrows, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
						staticTexts.add({staticLabel:"Columns:" } );
						var dual_a_maxcols_Editbox = realEditboxes.add( { editValue : ilp.dual_a_maxcols, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Second (right/bottom): Maximum:" } );			
						staticTexts.add({staticLabel:"Rows:" } );
						var dual_b_maxrows_Editbox = realEditboxes.add({ editValue : ilp.dual_b_maxrows, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
						staticTexts.add({staticLabel:"Columns:" } );
						var dual_b_maxcols_Editbox = realEditboxes.add( { editValue : ilp.dual_b_maxcols, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
					}
					*/
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Max rows or cols:" } );			
						var dual_b_max_Editbox = realEditboxes.add({ editValue : ilp.dual_b_max, minimumValue: 1, maximumValue: 25, minWidth: 30 } );
					}
					
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Left/Top Pane:   " } );
						var split = new Array();
						split[0] = checkboxControls.add( { staticLabel: "1/3   ", checkedState: ilp.split[0] } );	
						split[1] = split[2] = checkboxControls.add( { staticLabel: "equal   ", checkedState: ilp.split[1] } );	
						split[3] = checkboxControls.add( { staticLabel: "2/3", checkedState: ilp.split[3] } );
					}
					with( dialogRows.add () ) {		
						var orient = new Array();
						staticTexts.add({staticLabel:"Split:   " } );		
						orient[0] = checkboxControls.add( { staticLabel: "Horizontal   ", checkedState: ilp.orient[0] } );	
						orient[1] = checkboxControls.add( { staticLabel: "Vertical", checkedState: ilp.orient[1] } );	
					}
					with( dialogRows.add () ) {						
						staticTexts.add({staticLabel:"Aspect:   " } );
						staticTexts.add({staticLabel:"Min land: " } );
						var min_land = realEditboxes.add({ editValue: ilp.min_land, minimumValue: .1, maximumValue: 1, minWidth: 60 } );
						staticTexts.add({staticLabel:"   Max port: " } );
						var max_port = realEditboxes.add({ editValue: ilp.max_port, minimumValue: 1, maximumValue: 10, minWidth: 60 } );
					}
				}
			}
			with(borderPanels.add()){			
				var bleed_CheckBox = checkboxControls.add( { staticLabel: "Full Bleed", checkedState: ilp.bleed } );	
			}
	
			with(borderPanels.add()){
				with( dialogColumns.add() ) {  
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Gap between cells:" } );
					}
					with( dialogRows.add () ) {	
						var gapButtonGroup = radiobuttonGroups.add();				
						gapButtonGroup.radiobuttonControls.add( {staticLabel: "Relative to cell size" });
						gapButtonGroup.radiobuttonControls.add( {staticLabel: "Constant"});
						gapButtonGroup.selectedButton = ilp.fixed;
					}
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Relative (% of cell size):"});
						var gap_rel_Editbox    = percentEditboxes.add({	
																		editValue: ilp.gap_rel, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});
					}
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Constant (measurement units):"});
						var gap_fixed_Editbox = measurementEditboxes.add({ 	
																			editUnits: orig_hori_measurement_units,
																			editValue: ilp.gap_fixed, 
																			minWidth: 100, 
																			minimumValue: 0, 
																			maximumValue: 216
																		});
					}
				}
			}
		}
		
		// WRW 17 Nov 2011 Convert to two columns for better fit on short screens.
		
		with( dialogColumns.add() ) {   // Add a dialog column. 
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 						
					with( dialogRows.add () ) {	
						var caption_CheckBox = checkboxControls.add( { staticLabel: "Include Caption Text Frame", checkedState: ilp.caption } );	
					}
					
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Caption Height:"});
						var caption_y_Editbox = measurementEditboxes.add({	editValue: ilp.caption_y, 
																			minimumValue: 0, 
																			maximumValue: 100, 
																			minWidth: 50});
					}
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Caption Gap:"});
						var caption_gap_y_Editbox = measurementEditboxes.add({	editValue: ilp.caption_gap_y, 
																				minimumValue: 0, 
																				maximumValue: 100, 
																				minWidth: 50});					
					}			
				}
			}
			
			// -----------------------------------------------------------------------------------
			//	Scale & Alignment
			
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Layout scale (% available): " } );
						var scale_pc_Editbox = percentEditboxes.add({	editValue: ilp.scale_pc, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});	
					}

					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Near margin (% diagonal): " } );
						var near_margin_pc_Editbox = percentEditboxes.add({	editValue: ilp.near_margin_pc, 
																		minimumValue: 0, 
																		maximumValue: 100, 
																		minWidth: 50});	
					}
					
					var halign = new Array();
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Hori Alignment:   " } );		
						halign[0] = checkboxControls.add( { staticLabel: "left   ", checkedState: ilp.halign[0] } );
						halign[1] = checkboxControls.add( { staticLabel: "near left   ", checkedState: ilp.halign[1] } );
						halign[2] = checkboxControls.add( { staticLabel: "center   ", checkedState: ilp.halign[2] } );
						halign[3] = checkboxControls.add( { staticLabel: "near right   ", checkedState: ilp.halign[3] } );
						halign[4] = checkboxControls.add( { staticLabel: "right   ", checkedState: ilp.halign[4] } );
					}
					
					var valign = new Array();
					with( dialogRows.add () ) {		
						staticTexts.add({staticLabel:"Vert Alignment:   " } );		
						valign[0] = checkboxControls.add( { staticLabel: "top   ", checkedState: ilp.valign[0] } );
						valign[1] = checkboxControls.add( { staticLabel: "near top   ", checkedState: ilp.valign[1] } );
						valign[2] = checkboxControls.add( { staticLabel: "center   ", checkedState: ilp.valign[2] } );
						valign[3] = checkboxControls.add( { staticLabel: "near bottom   ", checkedState: ilp.valign[3] } );
						valign[4] = checkboxControls.add( { staticLabel: "bottom   ", checkedState: ilp.valign[4] } );
					}
				}
			}
			// -----------------------------------------------------------------------------------
			
			with(borderPanels.add( )) {			
				with( dialogColumns.add() ) { 					
					with( dialogRows.add () ) {	
						staticTexts.add({staticLabel:"Aspect Long:"});
						var aspect_long_Editbox = realEditboxes.add({editValue: ilp.aspect_long});
						staticTexts.add({staticLabel:"Short:"});
						var aspect_short_Editbox = realEditboxes.add({editValue: ilp.aspect_short});					
					}
				}
			}
		}
	}
	
	// -----------------------------------------------------------------------------------
	
	var myResult = myDialog.show();
	
	if( myResult === true ) {
		var i;
		var res = new Object();
		
		res.single_pane = single_CheckBox.checkedState;
		
		res.single_maxrows = single_maxrows_Editbox.editValue;
		res.single_maxcols = single_maxcols_Editbox.editValue;
		
		res.dual_pane = dual_CheckBox.checkedState;
		/*
		res.dual_a_maxrows = dual_a_maxrows_Editbox.editValue;
		res.dual_a_maxcols = dual_a_maxcols_Editbox.editValue;
		
		res.dual_b_maxrows = dual_b_maxrows_Editbox.editValue;
		res.dual_b_maxcols = dual_b_maxcols_Editbox.editValue;
		*/
		res.dual_b_max = dual_b_max_Editbox.editValue;	
		
		res.split = new Array();
		for( i = 0; i < 4; i++ ) res.split[i] = split[i].checkedState;	
		
		res.orient = new Array();
		for( i = 0; i < 2; i++ ) res.orient[i] = orient[i].checkedState;	
		
		res.bleed = bleed_CheckBox.checkedState;
		
		res.gap_fixed = gap_fixed_Editbox.editValue;
		res.gap_rel = gap_rel_Editbox.editValue;
		res.fixed = gapButtonGroup.selectedButton;
		
		res.caption = caption_CheckBox.checkedState;
		
		res.caption_y = caption_y_Editbox.editValue;
		res.caption_gap_y = caption_gap_y_Editbox.editValue;

		res.aspect_long = aspect_long_Editbox.editValue;
		res.aspect_short = aspect_short_Editbox.editValue;
		
		res.scale_pc = scale_pc_Editbox.editValue;
		res.near_margin_pc = near_margin_pc_Editbox.editValue;
		
		res.min_land = min_land.editValue;
		res.max_port = max_port.editValue;
		
		res.halign = new Array();
		res.valign = new Array();
		for( i = 0; i < 5; i++ ) {
			res.halign[i] = halign[i].checkedState;
			res.valign[i] = valign[i].checkedState;
		}
		
		res.shape = new Array();
		for( i = 0; i < 4; i++ ) res.shape[i] = shape[i].checkedState;
		
		for( i in res ) ilp[i] = res[i];
		
		r = ilp;
	}
	else {
		r = false;
	}

	myDialog.destroy();   //Remove the dialog box from memory.
	return r;
}

// *************************************************************************************************
//	Set up initial values for layout menu parameters
//		Note: Remember to run Photobook_Reset.jsx after changing this during a testing session.

function init_params() {
	this.aspect_mode = 0;
	this.aspect_long = 3872;
	this.aspect_short = 2592;
	this.rows = 2;
	this.cols = 2;
	this.scale_pc = 100;
	this.near_margin_pc = 8;
	this.halign = 2;
	this.valign = 2;
	this.gap_fixed = 36;
	this.gap_rel = 10;
	this.fixed = 0;
	this.caption = 0;
	this.caption_y = 20;
	this.caption_gap_y = 5;
	this.pane_dual = 0;				// 0 single, 1 dual
	this.pane_orient = 0;			// 0 Horizontal, 1 Vertical
	this.pane_split_button = 0;
	this.pane_custom = .25;
	this.pane_rows = 2;
	this.pane_cols = 1;
	this.guides = 0;
	this.bleedTop = 0;
	this.bleedRight = 0;
	this.bleedBottom = 0;
	this.bleedLeft = 0;
}

// *************************************************************************************************
// 	Set up initial values for library menu

function init_library_params() {
	this.single_pane = 1;		
	this.single_maxrows = 3;
	this.single_maxcols = 3;	
	this.dual_pane = 1;	
	/*
	this.dual_a_maxrows = 1;
	this.dual_a_maxcols = 1;
	this.dual_b_maxrows = 3;
	this.dual_b_maxcols = 3;
	*/
	this.dual_b_max = 3;
	this.split = [1, 1, 1, 1];	// Large on Right/Bottom, Even,  Even, Left/Top	
	this.orient = [1, 1];		// Horizontal, Vertical
	this.bleed = 1;	
	this.gap_fixed = 36;
	this.gap_rel = 10;
	this.fixed = 0;	
	this.caption = 0;
	this.caption_y = 20;
	this.caption_gap_y = 5;
	this.aspect_long = 3872;
	this.aspect_short = 2592;
	
	this.min_land = .1;
	this.max_port = 10;
	
	this.scale_pc = 100;
	this.near_margin_pc = 8;
	this.halign = [0, 0, 1, 0, 0];	// left, near left, center, near right, right
	this.valign = [0, 0, 1, 0, 0];	// top, near top, center, near bottom, bottom
	this.shape = [1, 1, 1, 1];		// Full, landscape, portrait, square
}

// *************************************************************************************************	
	
	
	
